
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "gedit-debug.h" */


GType gedit_debug_section_get_type (void);
#define GEDIT_TYPE_DEBUG_SECTION (gedit_debug_section_get_type())

/* enumerations from "gedit-tab.h" */


GType gedit_tab_state_get_type (void);
#define GEDIT_TYPE_TAB_STATE (gedit_tab_state_get_type())

/* enumerations from "gedit-window.h" */


GType gedit_window_state_get_type (void);
#define GEDIT_TYPE_WINDOW_STATE (gedit_window_state_get_type())

G_END_DECLS

/* Generated data ends here */

